'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.fluxEnhancer = fluxEnhancer;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _objectMerge = require('object-merge');

var _objectMerge2 = _interopRequireDefault(_objectMerge);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Creates a store enhancer which augments the store's state to include keys
 * and values specified by Store's similar to those used in traditional Flux
 * @param {Object} storeMap - A mapping of a property key to an object which contains a `reduce(state, action, waitFor)` method
*/
function fluxEnhancer(storeMap) {
  return function (storeCreator) {
    return function (inputReducer, initialState) {
      var reducer = inputReducer || function () {
        return {};
      };

      // Map stores -> keys for easy access in reduce
      var storesToKeys = _lodash2.default.reduce(storeMap, function (result, store, key) {
        return result.set(store, key);
      }, new Map());

      // This allows Stores to call getState() on a different store
      // that it depends on in the middle of its own reduction, using
      // the other store's value after it reduces the action.
      var partiallyReducedState = null;

      // Runs an action through all the stores and returns a new state
      // variable with the new state of all the keys in storeMap
      var reduceFromStores = function reduceFromStores() {
        var state = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
        var action = arguments[1];

        var newState = {};
        var completedSet = new Set();
        partiallyReducedState = Object.assign({}, state);

        // Var is used since assignKey and waitFor call each other
        var waitFor = function waitFor(stores) {
          // eslint-disable-line
          _lodash2.default.forEach(stores, function (store) {
            return assignKey(storesToKeys.get(store));
          }); // eslint-disable-line
        };

        var assignKey = function assignKey(key) {
          // eslint-disable-line
          if (completedSet.has(key)) return;
          var store = storeMap[key];
          newState[key] = store.reduce.bind(store)(state[key], action, waitFor);
          partiallyReducedState[key] = newState[key];
          completedSet.add(key);
        };

        _lodash2.default.forEach(Object.keys(storeMap), assignKey);

        partiallyReducedState = null;
        return newState;
      };

      var augmentedReducer = function augmentedReducer(state, action) {
        var storeResult = reduceFromStores(state, action);
        var reducerResult = reducer(state, action);
        return (0, _objectMerge2.default)(reducerResult, storeResult);
      };

      var store = storeCreator(augmentedReducer, initialState);
      var storeDotGetState = store.getState;
      store.getState = function () {
        return partiallyReducedState || storeDotGetState();
      };

      return store;
    };
  };
}